#!/usr/bin/env python

import ConfigParser
import datetime
import glob
import os
import re
import sys


class IRCParser:
    def __init__(self,
                 config,
                 last_days,
                 longest_response,
                 day_start,
                 day_end,
                 tracking):

        # use config for object
        self.config = config

        # analyze the last x days
        self.last_days = last_days

        # ignore longer reponses
        self.longest_response = longest_response

        # day start and end hours
        self.day_start = day_start
        self.day_end = day_end

        # tracking words
        self.tracking = tracking

        # main questions dictionary
        self.questions = {}

        # counters(overall, day, night)
        self.questions_num = [0, 0, 0]

        # output which goes to logs
        self.output = []

        # sums(overlall, day, night)
        self.reaction_sum = [datetime.timedelta(),
                             datetime.timedelta(),
                             datetime.timedelta()]
        self.reaction_max = [datetime.timedelta(),
                             datetime.timedelta(),
                             datetime.timedelta()]
        self.reaction_min = [datetime.timedelta(days=999),
                             datetime.timedelta(days=999),
                             datetime.timedelta(days=999)]

    def run(self):
        # read log files here
        files = glob.glob("%s*" % self.config['location'])
        files.sort(key=os.path.getmtime)
        for log in files:
            # parse filename
            date_text = log[-12:-4]
            # read file
            for line in open(log).readlines():
                # only parse an actual chat lines
                if '<' not in line:
                    continue
                # make parsable values
                time_text = line[1:9]
                date = datetime.datetime.strptime(
                    '%s-%s' % (date_text, time_text),
                    '%Y%m%d-%H:%M:%S')
                # skip if message too old to process
                if datetime.datetime.now() - date > datetime.timedelta(
                                                    days=self.last_days):
                    continue
                nick = re.match('.*?<(.*?)>.*', line).group(1)
                text = re.match('.*?<.*?> (.*)', line).group(1)
                target_match = re.match('.*?<.*?> @?(\S*):.*', line)
                if target_match:
                    target = target_match.group(1)
                else:
                    target = None
                # run qa parser function
                self.qa_parser(channel, date, nick, target, text)

    # question/answer parser
    def qa_parser(self, channel, date, nick, target, text):
        # cleanup nick
        nick = nick.lower().replace('(bot)', '')
        # look for a question or request
        if nick not in self.config['ops'] and self.tracking_search(text):
            q_id = len(self.questions)
            day = date.hour in range(self.day_start, self.day_end)
            self.questions[q_id] = {'nick': nick,
                                    'date': date,
                                    'day': day,
                                    'question': text,
                                    'answered': False}
        # look for answers
        if nick in self.config['ops']:
            for x in self.questions.keys():
                if self.questions[x]['nick'] == target \
                   and not self.questions[x]['answered']:
                    spend = date - self.questions[x]['date']
                    # skip if response time seems invalid
                    if spend > datetime.timedelta(hours=self.longest_response):
                        continue
                    self.questions[x]['answered'] = True
                    self.questions[x]['reaction'] = spend
                    self.output.append('Question: %s %s %s<br>' %
                                       (self.questions[x]['date'],
                                        self.questions[x]['nick'],
                                        self.questions[x]['question']))
                    self.output.append('Answer: %s %s %s<br>' %
                                       (date, nick, text))
                    self.output.append('Reaction time %s<br>' % spend)
                    self.output.append('<hr>')

    def stats(self):
        for question in self.questions.iteritems():
            # strip tuple to dictionary
            question = question[1]
            # calculate overall
            self.parse_response(0, question)
            if question['day']:
                # day time
                self.parse_response(1, question)
            else:
                # night time
                self.parse_response(2, question)

        if self.questions_num[0]:
            average_overall = self.reaction_sum[0] / self.questions_num[0]
        else:
            average_overall = datetime.timedelta(days=999)

        if self.questions_num[1]:
            average_day = self.reaction_sum[1] / self.questions_num[1]
        else:
            average_day = datetime.timedelta(days=999)

        if self.questions_num[2]:
            average_night = self.reaction_sum[2] / self.questions_num[2]
        else:
            average_night = datetime.timedelta(days=999)

        self.output.append('General statistics - updated %s<br>' %
                           datetime.datetime.now())
        self.output.append(
            'Questions counter (overlay, day, night): %s, %s, %s<br>' %
            (self.questions_num[0],
             self.questions_num[1],
             self.questions_num[2]))
        self.output.append(
            'Average reaction time (overlay, day, night): %s, %s, %s<br>' %
            (average_overall, average_day, average_night))
        self.output.append(
            'The shortest reaction time (overlay, day, night): %s, %s, %s<br>'
            % (self.reaction_min[0],
               self.reaction_min[1],
               self.reaction_min[2]))
        self.output.append(
            'The longest reaction time (overlay, day, night): %s, %s, %s<br>'
            % (self.reaction_max[0],
               self.reaction_max[1],
               self.reaction_max[2]))

        f = open("%s/%s.html" % (self.config['output'], channel), 'w')
        for line in self.output:
            f.write(line)
        f.close()

    def parse_response(self, field, question):
        if 'reaction' in question:
            self.questions_num[field] += 1
            self.reaction_sum[field] += question['reaction']
            if self.reaction_min[field] > question['reaction']:
                self.reaction_min[field] = question['reaction']
            if self.reaction_max[field] < question['reaction']:
                self.reaction_max[field] = question['reaction']

    # quick search for tracking words
    def tracking_search(self, text):
        for k in self.tracking:
            if k in text:
                return True
        return False

if len(sys.argv) != 2:
    print 'Usage: ircparser.py <config>'
    sys.exit(1)

# read config and run parser for every channel
config = ConfigParser.ConfigParser()
try:
    config.readfp(open(sys.argv[1]))
except IOError as e:
    print 'Config file is unreadable: %s' % e
    sys.exit(1)

# read global configuration
last_days = config.getint('DEFAULT', 'last_days')
longest_response = config.getint('DEFAULT', 'longest_response')
day_start = config.getint('DEFAULT', 'day_start')
day_end = config.getint('DEFAULT', 'day_end')
tracking = config.get('DEFAULT', 'tracking').split(',')

for channel in config.sections():
    ops = map(lambda x: x.lower(), config.get(channel, 'ops').split(','))
    chan_config = {'location': config.get(channel, 'location'),
                   'output': config.get(channel, 'output'),
                   'ops': ops}

    ircparser = IRCParser(chan_config,
                          last_days,
                          longest_response,
                          day_start,
                          day_end,
                          tracking)
    ircparser.run()
    ircparser.stats()
